@@ -1,10 +1,50 @@
 # encoding: utf-8
 """Word completion for IPython.
 
-This module started as fork of the rlcompleter module in the Python standard
+This module is a fork of the rlcompleter module in the Python standard
 library.  The original enhancements made to rlcompleter have been sent
-upstream and were accepted as of Python 2.3,
+upstream and were accepted as of Python 2.3, but we need a lot more
+functionality specific to IPython, so this module will continue to live as an
+IPython-specific utility.
 
+Original rlcompleter documentation:
+
+This requires the latest extension to the readline module (the
+completes keywords, built-ins and globals in __main__; when completing
+NAME.NAME..., it evaluates (!) the expression up to the last dot and
+completes its attributes.
+
+It's very cool to do "import string" type "string.", hit the
+completion key (twice), and see the list of names defined by the
+string module!
+
+Tip: to use the tab key as the completion key, call
+
+    readline.parse_and_bind("tab: complete")
+
+Notes:
+
+- Exceptions raised by the completer function are *ignored* (and
+  generally cause the completion to fail).  This is a feature -- since
+  readline sets the tty device in raw (or cbreak) mode, printing a
+  traceback wouldn't work well without some complicated hoopla to save,
+  reset and restore the tty state.
+
+- The evaluation of the NAME.NAME... form may cause arbitrary
+  application defined code to be executed if an object with a
+  ``__getattr__`` hook is found.  Since it is the responsibility of the
+  application (or the user) to enable this feature, I consider this an
+  acceptable risk.  More complicated expressions (e.g. function calls or
+  indexing operations) are *not* evaluated.
+
+- GNU readline is also used by the built-in functions input() and
+  raw_input(), and thus these also benefit/suffer from the completer
+  features.  Clearly an interactive application can benefit by
+  specifying its own completer function and using raw_input() for all
+  its input.
+
+- When the original stdin is not a tty device, GNU readline is never
+  used, and this module (and the readline module) are silently inactive.
 """
 
 # Copyright (c) IPython Development Team.
@@ -13,8 +53,6 @@ upstream and were accepted as of Python 2.3,
 # Some of this code originated from rlcompleter in the Python standard library
 # Copyright (C) 2001 Python Software Foundation, www.python.org
 
-from __future__ import print_function
-
 import __main__
 import glob
 import inspect
@@ -25,19 +63,22 @@ import re
 import sys
 import unicodedata
 import string
-import warnings
 
-from traitlets.config.configurable import Configurable
+from traitlets.config.configurable import Configurable 
 from IPython.core.error import TryNext
 from IPython.core.inputsplitter import ESC_MAGIC
 from IPython.core.latex_symbols import latex_symbols, reverse_latex_symbol
 from IPython.utils import generics
+from IPython.utils import io
 from IPython.utils.decorators import undoc
-from IPython.utils.dir2 import dir2, get_real_method
+from IPython.utils.dir2 import dir2
 from IPython.utils.process import arg_split
-from IPython.utils.py3compat import builtin_mod, string_types, PY3, cast_unicode_py2
-from traitlets import Bool, Enum, observe
+from IPython.utils.py3compat import builtin_mod, string_types, PY3
+from traitlets import CBool, Enum
 
+#-----------------------------------------------------------------------------
+# Globals
+#-----------------------------------------------------------------------------
 
 # Public API
 __all__ = ['Completer','IPCompleter']
@@ -48,6 +89,10 @@ else:
     PROTECTABLES = ' ()[]{}?=\\|;:\'#*"^&'
 
 
+#-----------------------------------------------------------------------------
+# Main functions and classes
+#-----------------------------------------------------------------------------
+
 def has_open_quotes(s):
     """Return whether a string has open quotes.
 
@@ -71,14 +116,9 @@ def has_open_quotes(s):
 
 def protect_filename(s):
     """Escape a string to protect certain characters."""
-    if set(s) & set(PROTECTABLES):
-        if sys.platform == "win32":
-            return '"' + s + '"'
-        else:
-            return "".join(("\\" + c if c in PROTECTABLES else c) for c in s)
-    else:
-        return s
 
+    return "".join([(ch in PROTECTABLES and '\\' + ch or ch)
+                    for ch in s])
 
 def expand_user(path):
     """Expand '~'-style usernames in strings.
@@ -129,52 +169,49 @@ def compress_user(path, tilde_expand, tilde_val):
         return path
 
 
-def completions_sorting_key(word):
-    """key for sorting completions
 
-    This does several things:
+def penalize_magics_key(word):
+    """key for sorting that penalizes magic commands in the ordering
+
+    Normal words are left alone.
+
+    Magic commands have the initial % moved to the end, e.g.
+    %matplotlib is transformed as follows:
+
+    %matplotlib -> matplotlib%
+
+    [The choice of the final % is arbitrary.]
+
+    Since "matplotlib" < "matplotlib%" as strings, 
+    "timeit" will appear before the magic "%timeit" in the ordering
+
+    For consistency, move "%%" to the end, so cell magics appear *after*
+    line magics with the same name.
+
+    A check is performed that there are no other "%" in the string; 
+    if there are, then the string is not a magic command and is left unchanged.
 
-    - Lowercase all completions, so they are sorted alphabetically with
-      upper and lower case words mingled
-    - Demote any completions starting with underscores to the end
-    - Insert any %magic and %%cellmagic completions in the alphabetical order
-      by their name
     """
-    # Case insensitive sort
-    word = word.lower()
 
-    prio1, prio2 = 0, 0
+    # Move any % signs from start to end of the key 
+    # provided there are no others elsewhere in the string
 
-    if word.startswith('__'):
-        prio1 = 2
-    elif word.startswith('_'):
-        prio1 = 1
-
-    if word.endswith('='):
-        prio1 = -1
-
-    if word.startswith('%%'):
-        # If there's another % in there, this is something else, so leave it alone
+    if word[:2] == "%%":
         if not "%" in word[2:]:
-            word = word[2:]
-            prio2 = 2
-    elif word.startswith('%'):
-        if not "%" in word[1:]:
-            word = word[1:]
-            prio2 = 1
+            return word[2:] + "%%" 
 
-    return prio1, word, prio2
+    if word[:1] == "%":
+        if not "%" in word[1:]:
+            return word[1:] + "%"
+    
+    return word
 
 
 @undoc
 class Bunch(object): pass
 
 
-if sys.platform == 'win32':
-    DELIMS = ' \t\n`!@#$^&*()=+[{]}|;\'",<>?'
-else:
-    DELIMS = ' \t\n`!@#$^&*()=+[{]}\\|;:\'",<>?'
-
+DELIMS = ' \t\n`!@#$^&*()=+[{]}\\|;:\'",<>?'
 GREEDY_DELIMS = ' =\r\n'
 
 
@@ -231,20 +268,19 @@ class CompletionSplitter(object):
 
 class Completer(Configurable):
 
-    greedy = Bool(False,
+    greedy = CBool(False, config=True,
         help="""Activate greedy completion
-        PENDING DEPRECTION. this is now mostly taken care of with Jedi.
 
         This will enable completion on elements of lists, results of function calls, etc.,
         but can be unsafe because the code is actually evaluated on TAB.
         """
-    ).tag(config=True)
+    )
     
 
     def __init__(self, namespace=None, global_namespace=None, **kwargs):
         """Create a new completer for the command line.
 
-        Completer(namespace=ns, global_namespace=ns2) -> completer instance.
+        Completer(namespace=ns,global_namespace=ns2) -> completer instance.
 
         If unspecified, the default namespace where completions are performed
         is __main__ (technically, __main__.__dict__). Namespaces should be
@@ -304,6 +340,7 @@ class Completer(Configurable):
         defined in self.namespace or self.global_namespace that match.
 
         """
+        #print 'Completer->global_matches, txt=%r' % text # dbg
         matches = []
         match_append = matches.append
         n = len(text)
@@ -314,7 +351,7 @@ class Completer(Configurable):
             for word in lst:
                 if word[:n] == text and word != "__builtins__":
                     match_append(word)
-        return [cast_unicode_py2(m) for m in matches]
+        return matches
 
     def attr_matches(self, text):
         """Compute matches when text contains a dot.
@@ -330,6 +367,7 @@ class Completer(Configurable):
 
         """
 
+        #io.rprint('Completer->attr_matches, txt=%r' % text) # dbg
         # Another option, seems to work great. Catches things like ''.<tab>
         m = re.match(r"(\S+(\.\w+)*)\.(\w*)$", text)
     
@@ -366,7 +404,8 @@ class Completer(Configurable):
             pass
         # Build match list to return
         n = len(attr)
-        return [u"%s.%s" % (expr, w) for w in words if w[:n] == attr ]
+        res = ["%s.%s" % (expr, w) for w in words if w[:n] == attr ]
+        return res
 
 
 def get__all__entries(obj):
@@ -376,7 +415,7 @@ def get__all__entries(obj):
     except:
         return []
     
-    return [cast_unicode_py2(w) for w in words if isinstance(w, string_types)]
+    return [w for w in words if isinstance(w, string_types)]
 
 
 def match_dict_keys(keys, prefix, delims):
@@ -437,6 +476,7 @@ def _safe_isinstance(obj, module, class_name):
             isinstance(obj, getattr(__import__(module), class_name)))
 
 
+
 def back_unicode_name_matches(text):
     u"""Match unicode characters back to unicode name
     
@@ -463,7 +503,7 @@ def back_unicode_name_matches(text):
     try :
         unic = unicodedata.name(char)
         return '\\'+char,['\\'+unic]
-    except KeyError:
+    except KeyError as e:
         pass
     return u'', ()
 
@@ -490,18 +530,17 @@ def back_latex_name_matches(text):
         latex = reverse_latex_symbol[char]
         # '\\' replace the \ as well
         return '\\'+char,[latex]
-    except KeyError:
+    except KeyError as e:
         pass
     return u'', ()
 
 
 class IPCompleter(Completer):
     """Extension of the completer class with IPython-specific features"""
-    
-    @observe('greedy')
-    def _greedy_changed(self, change):
+
+    def _greedy_changed(self, name, old, new):
         """update the splitter and readline delims when greedy is changed"""
-        if change['new']:
+        if new:
             self.splitter.delims = GREEDY_DELIMS
         else:
             self.splitter.delims = DELIMS
@@ -509,14 +548,14 @@ class IPCompleter(Completer):
         if self.readline:
             self.readline.set_completer_delims(self.splitter.delims)
     
-    merge_completions = Bool(True,
+    merge_completions = CBool(True, config=True,
         help="""Whether to merge completion results into a single list
         
         If False, only the completion results from the first non-empty
         completer will be returned.
         """
-    ).tag(config=True)
-    omit__names = Enum((0,1,2), default_value=2,
+    )
+    omit__names = Enum((0,1,2), default_value=2, config=True,
         help="""Instruct the completer to omit private method names
         
         Specifically, when completing on ``object.<tab>``.
@@ -527,27 +566,17 @@ class IPCompleter(Completer):
         
         When 0: nothing will be excluded.
         """
-    ).tag(config=True)
-    limit_to__all__ = Bool(False,
-        help="""
-        DEPRECATED as of version 5.0.
-        
-        Instruct the completer to use __all__ for the completion
+    )
+    limit_to__all__ = CBool(default_value=False, config=True,
+        help="""Instruct the completer to use __all__ for the completion
         
         Specifically, when completing on ``object.<tab>``.
         
         When True: only those names in obj.__all__ will be included.
         
         When False [default]: the __all__ attribute is ignored 
-        """,
-    ).tag(config=True)
-
-    @observe('limit_to__all__')
-    def _limit_to_all_changed(self, change):
-        warnings.warn('`IPython.core.IPCompleter.limit_to__all__` configuration '
-            'value has been deprecated since IPython 5.0, will be made to have '
-            'no effects and then removed in future version of IPython.',
-            UserWarning)
+        """
+    )
 
     def __init__(self, shell=None, namespace=None, global_namespace=None,
                  use_readline=True, config=None, **kwargs):
@@ -614,24 +643,21 @@ class IPCompleter(Completer):
         #= re.compile(r'[\s|\[]*(\w+)(?:\s*=?\s*.*)')
 
         # All active matcher routines for completion
-        self.matchers = [
-                         self.python_matches,
+        self.matchers = [self.python_matches,
                          self.file_matches,
                          self.magic_matches,
                          self.python_func_kw_matches,
                          self.dict_key_matches,
                          ]
 
-        # This is set externally by InteractiveShell
-        self.custom_completers = None
-
     def all_completions(self, text):
         """
-        Wrapper around the complete method for the benefit of emacs.
+        Wrapper around the complete method for the benefit of emacs
+        and pydb.
         """
         return self.complete(text)[1]
 
-    def _clean_glob(self, text):
+    def _clean_glob(self,text):
         return self.glob("%s*" % text)
 
     def _clean_glob_win32(self,text):
@@ -652,15 +678,17 @@ class IPCompleter(Completer):
         current (as of Python 2.3) Python readline it's possible to do
         better."""
 
+        #io.rprint('Completer->file_matches: <%r>' % text) # dbg
+
         # chars that require escaping with backslash - i.e. chars
         # that readline treats incorrectly as delimiters, but we
         # don't want to treat as delimiters in filename matching
         # when escaped with backslash
         if text.startswith('!'):
             text = text[1:]
-            text_prefix = u'!'
+            text_prefix = '!'
         else:
-            text_prefix = u''
+            text_prefix = ''
 
         text_until_cursor = self.text_until_cursor
         # track strings with open quotes
@@ -691,13 +719,10 @@ class IPCompleter(Completer):
             text = os.path.expanduser(text)
 
         if text == "":
-            return [text_prefix + cast_unicode_py2(protect_filename(f)) for f in self.glob("*")]
+            return [text_prefix + protect_filename(f) for f in self.glob("*")]
 
         # Compute the matches from the filesystem
-        if sys.platform == 'win32':
-            m0 = self.clean_glob(text)
-        else:
-            m0 = self.clean_glob(text.replace('\\', ''))
+        m0 = self.clean_glob(text.replace('\\',''))
 
         if has_protectables:
             # If we had protectables, we need to revert our changes to the
@@ -716,11 +741,15 @@ class IPCompleter(Completer):
                 matches = [text_prefix +
                            protect_filename(f) for f in m0]
 
+        #io.rprint('mm', matches)  # dbg
+
         # Mark directories in input list by appending '/' to their names.
-        return [cast_unicode_py2(x+'/') if os.path.isdir(x) else x for x in matches]
+        matches = [x+'/' if os.path.isdir(x) else x for x in matches]
+        return matches
 
     def magic_matches(self, text):
         """Match magics"""
+        #print 'Completer->magic_matches:',text,'lb',self.text_until_cursor # dbg
         # Get all shell magics now rather than statically, so magics loaded at
         # runtime show up too.
         lsm = self.shell.magics_manager.lsmagic()
@@ -738,11 +767,12 @@ class IPCompleter(Completer):
         comp = [ pre2+m for m in cell_magics if m.startswith(bare_text)]
         if not text.startswith(pre2):
             comp += [ pre+m for m in line_magics if m.startswith(bare_text)]
-        return [cast_unicode_py2(c) for c in comp]
+        return comp
 
-
-    def python_matches(self, text):
+    def python_matches(self,text):
         """Match attributes or global python names"""
+        
+        #io.rprint('Completer->python_matches, txt=%r' % text) # dbg
         if "." in text:
             try:
                 matches = self.attr_matches(text)
@@ -761,6 +791,7 @@ class IPCompleter(Completer):
                 matches = []
         else:
             matches = self.global_matches(text)
+
         return matches
 
     def _default_arguments_from_docstring(self, doc):
@@ -889,19 +920,13 @@ class IPCompleter(Completer):
 
             for namedArg in namedArgs:
                 if namedArg.startswith(text):
-                    argMatches.append(u"%s=" %namedArg)
+                    argMatches.append("%s=" %namedArg)
         return argMatches
 
     def dict_key_matches(self, text):
         "Match string keys in a dictionary, after e.g. 'foo[' "
         def get_keys(obj):
-            # Objects can define their own completions by defining an
-            # _ipy_key_completions_() method.
-            method = get_real_method(obj, '_ipython_key_completions_')
-            if method is not None:
-                return method()
-
-            # Special case some common in-memory dict-like types
+            # Only allow completion for known in-memory dict-like types
             if isinstance(obj, dict) or\
                _safe_isinstance(obj, 'pandas', 'DataFrame'):
                 try:
@@ -1019,7 +1044,7 @@ class IPCompleter(Completer):
                 # allow combining chars
                 if ('a'+unic).isidentifier():
                     return '\\'+s,[unic]
-            except KeyError:
+            except KeyError as e:
                 pass
         return u'', []
 
@@ -1048,9 +1073,7 @@ class IPCompleter(Completer):
         return u'', []
 
     def dispatch_custom_completer(self, text):
-        if not self.custom_completers:
-            return
-
+        #io.rprint("Custom! '%s' %s" % (text, self.custom_completers)) # dbg
         line = self.line_buffer
         if not line.strip():
             return None
@@ -1064,6 +1087,8 @@ class IPCompleter(Completer):
         event.command = cmd
         event.text_until_cursor = self.text_until_cursor
 
+        #print "\ncustom:{%s]\n" % event # dbg
+
         # for foo etc, try also to find completer for %foo
         if not cmd.startswith(self.magic_escape):
             try_magic = self.custom_completers.s_matches(
@@ -1074,16 +1099,17 @@ class IPCompleter(Completer):
         for c in itertools.chain(self.custom_completers.s_matches(cmd),
                  try_magic,
                  self.custom_completers.flat_matches(self.text_until_cursor)):
+            #print "try",c # dbg
             try:
                 res = c(event)
                 if res:
                     # first, try case sensitive match
-                    withcase = [cast_unicode_py2(r) for r in res if r.startswith(text)]
+                    withcase = [r for r in res if r.startswith(text)]
                     if withcase:
                         return withcase
                     # if none, then case insensitive ones are ok too
                     text_low = text.lower()
-                    return [cast_unicode_py2(r) for r in res if r.lower().startswith(text_low)]
+                    return [r for r in res if r.lower().startswith(text_low)]
             except TryNext:
                 pass
 
@@ -1119,14 +1145,13 @@ class IPCompleter(Completer):
         matches : list
           A list of completion matches.
         """
+        # io.rprint('\nCOMP1 %r %r %r' % (text, line_buffer, cursor_pos))  # dbg
+
         # if the cursor position isn't given, the only sane assumption we can
         # make is that it's at the end of the line (the common case)
         if cursor_pos is None:
             cursor_pos = len(line_buffer) if text is None else len(text)
 
-        if self.use_main_ns:
-            self.namespace = __main__.__dict__
-
         if PY3:
 
             base_text = text if not line_buffer else line_buffer[:cursor_pos]
@@ -1150,6 +1175,7 @@ class IPCompleter(Completer):
 
         self.line_buffer = line_buffer
         self.text_until_cursor = self.line_buffer[:cursor_pos]
+        # io.rprint('COMP2 %r %r %r' % (text, line_buffer, cursor_pos))  # dbg
 
         # Start with a clean slate of completions
         self.matches[:] = []
@@ -1179,6 +1205,72 @@ class IPCompleter(Completer):
         # different types of objects.  The rlcomplete() method could then
         # simply collapse the dict into a list for readline, but we'd have
         # richer completion semantics in other evironments.
-        self.matches = sorted(set(self.matches), key=completions_sorting_key)
 
+        # use penalize_magics_key to put magics after variables with same name
+        self.matches = sorted(set(self.matches), key=penalize_magics_key)
+
+        #io.rprint('COMP TEXT, MATCHES: %r, %r' % (text, self.matches)) # dbg
         return text, self.matches
+
+    def rlcomplete(self, text, state):
+        """Return the state-th possible completion for 'text'.
+
+        This is called successively with state == 0, 1, 2, ... until it
+        returns None.  The completion should begin with 'text'.
+
+        Parameters
+        ----------
+          text : string
+            Text to perform the completion on.
+
+          state : int
+            Counter used by readline.
+        """
+        if state==0:
+
+            self.line_buffer = line_buffer = self.readline.get_line_buffer()
+            cursor_pos = self.readline.get_endidx()
+
+            #io.rprint("\nRLCOMPLETE: %r %r %r" %
+            #          (text, line_buffer, cursor_pos) ) # dbg
+
+            # if there is only a tab on a line with only whitespace, instead of
+            # the mostly useless 'do you want to see all million completions'
+            # message, just do the right thing and give the user his tab!
+            # Incidentally, this enables pasting of tabbed text from an editor
+            # (as long as autoindent is off).
+
+            # It should be noted that at least pyreadline still shows file
+            # completions - is there a way around it?
+
+            # don't apply this on 'dumb' terminals, such as emacs buffers, so
+            # we don't interfere with their own tab-completion mechanism.
+            if not (self.dumb_terminal or line_buffer.strip()):
+                self.readline.insert_text('\t')
+                sys.stdout.flush()
+                return None
+
+            # Note: debugging exceptions that may occur in completion is very
+            # tricky, because readline unconditionally silences them.  So if
+            # during development you suspect a bug in the completion code, turn
+            # this flag on temporarily by uncommenting the second form (don't
+            # flip the value in the first line, as the '# dbg' marker can be
+            # automatically detected and is used elsewhere).
+            DEBUG = False
+            #DEBUG = True # dbg
+            if DEBUG:
+                try:
+                    self.complete(text, line_buffer, cursor_pos)
+                except:
+                    import traceback; traceback.print_exc()
+            else:
+                # The normal production version is here
+
+                # This method computes the self.matches array
+                self.complete(text, line_buffer, cursor_pos)
+
+        try:
+            return self.matches[state]
+        except IndexError:
+            return None
+
