@@ -1,7 +1,6 @@
 """Implementation of code management magic functions.
 """
 from __future__ import print_function
-from __future__ import absolute_import
 #-----------------------------------------------------------------------------
 #  Copyright (c) 2012 The IPython Development Team.
 #
@@ -32,9 +31,8 @@ from IPython.testing.skipdoctest import skip_doctest
 from IPython.utils import py3compat
 from IPython.utils.py3compat import string_types
 from IPython.utils.contexts import preserve_keys
-from IPython.utils.path import get_py_filename
-from warnings import warn
-from logging import error
+from IPython.utils.path import get_py_filename, unquote_filename
+from IPython.utils.warn import warn, error
 from IPython.utils.text import get_text_list
 
 #-----------------------------------------------------------------------------
@@ -138,36 +136,6 @@ def extract_symbols(code, symbols):
 
     return blocks, not_found
 
-def strip_initial_indent(lines):
-    """For %load, strip indent from lines until finding an unindented line.
-
-    https://github.com/ipython/ipython/issues/9775
-    """
-    indent_re = re.compile(r'\s+')
-
-    it = iter(lines)
-    first_line = next(it)
-    indent_match = indent_re.match(first_line)
-
-    if indent_match:
-        # First line was indented
-        indent = indent_match.group()
-        yield first_line[len(indent):]
-
-        for line in it:
-            if line.startswith(indent):
-                yield line[len(indent):]
-            else:
-                # Less indented than the first line - stop dedenting
-                yield line
-                break
-    else:
-        yield first_line
-
-    # Pass the remaining lines through without dedenting
-    for line in it:
-        yield line
-
 
 class InteractivelyDefined(Exception):
     """Exception for interactively defined variable in magic_edit"""
@@ -219,7 +187,7 @@ class CodeMagics(Magics):
         append = 'a' in opts
         mode = 'a' if append else 'w'
         ext = u'.ipy' if raw else u'.py'
-        fname, codefrom = args[0], " ".join(args[1:])
+        fname, codefrom = unquote_filename(args[0]), " ".join(args[1:])
         if not fname.endswith((u'.py',u'.ipy')):
             fname += ext
         file_exists = os.path.isfile(fname)
@@ -371,7 +339,7 @@ class CodeMagics(Magics):
             lines = contents.split('\n')
             slices = extract_code_ranges(ranges)
             contents = [lines[slice(*slc)] for slc in slices]
-            contents = '\n'.join(strip_initial_indent(chain.from_iterable(contents)))
+            contents = '\n'.join(chain.from_iterable(contents))
 
         l = len(contents)
 
@@ -399,6 +367,7 @@ class CodeMagics(Magics):
 
         def make_filename(arg):
             "Make a filename from the given args"
+            arg = unquote_filename(arg)
             try:
                 filename = get_py_filename(arg)
             except IOError:
