@@ -25,6 +25,8 @@ import logging
 import os
 import re
 import sys
+import traceback
+import unittest
 
 from testpath import modified_env
 
@@ -39,9 +41,10 @@ from doctest import (REPORTING_FLAGS, REPORT_ONLY_FIRST_FAILURE,
                      linecache)
 
 # Third-party modules
+import nose.core
 
 from nose.plugins import doctests, Plugin
-from nose.util import anyp, tolist
+from nose.util import anyp, getpackage, test_address, resolve_name, tolist
 
 # Our own imports
 from IPython.utils.py3compat import builtin_mod, PY3, getcwd
@@ -140,7 +143,7 @@ class DocTestFinder(doctest.DocTestFinder):
         # doctests in extension modules.
 
         # Local shorthands
-        from inspect import isroutine, isclass
+        from inspect import isroutine, isclass, ismodule
 
         # Look for tests in a module's contained objects.
         if inspect.ismodule(obj) and self._recurse:
