@@ -22,7 +22,7 @@ from traitlets import (
     Union, All, Undefined, Type, This, Instance, TCPAddress, List, Tuple,
     ObjectName, DottedObjectName, CRegExp, link, directional_link,
     ForwardDeclaredType, ForwardDeclaredInstance, validate, observe, default,
-    observe_compat,
+    observe_compat, BaseDescriptor, HasDescriptors,
 )
 from ipython_genutils import py3compat
 from ipython_genutils.testing.decorators import skipif
@@ -266,6 +266,25 @@ class TestHasDescriptorsMeta(TestCase):
         self.assertEqual(B.tt.this_class, B)
         self.assertEqual(B.ttt.this_class, B)
 
+class TestHasDescriptors(TestCase):
+
+    def test_setup_instance(self):
+
+        class FooDescriptor(BaseDescriptor):
+
+            def instance_init(self, inst):
+                foo = inst.foo # instance should have the attr
+
+        class HasFooDescriptors(HasDescriptors):
+
+            fd = FooDescriptor()
+
+            def setup_instance(self, *args, **kwargs):
+                self.foo = kwargs.get('foo', None)
+                super(HasFooDescriptors, self).setup_instance(*args, **kwargs)
+
+        hfd = HasFooDescriptors(foo='bar')
+
 class TestHasTraitsNotify(TestCase):
 
     def setUp(self):
@@ -1025,7 +1044,7 @@ class TestThis(TestCase):
 
         tree = Tree(
             value='foo',
-            leaves=[Tree('bar'), Tree('buzz')]
+            leaves=[Tree(value='bar'), Tree(value='buzz')]
         )
 
         with self.assertRaises(TraitError):
@@ -2163,3 +2182,33 @@ def test_subclass_add_observer():
     obj.trait = 5
     nt.assert_true(obj.child_called)
     nt.assert_true(obj.parent_called)
+
+def test_super_args():
+    class SuperRecorder(object):
+        def __init__(self, *args, **kwargs):
+            self.super_args = args
+            self.super_kwargs = kwargs
+    
+    class SuperHasTraits(HasTraits, SuperRecorder):
+        i = Integer()
+    
+    obj = SuperHasTraits('a1', 'a2', b=10, i=5, c='x')
+    nt.assert_equal(obj.i, 5)
+    assert not hasattr(obj, 'b')
+    assert not hasattr(obj, 'c')
+    nt.assert_equal(obj.super_args, ('a1', 'a2'))
+    nt.assert_equal(obj.super_kwargs, {'b': 10, 'c': 'x'})
+
+def test_super_bad_args():
+    class SuperHasTraits(HasTraits):
+        a = Integer()
+    
+    if sys.version_info < (3,):
+        # Legacy Python, object.__init__ warns itself, instead of raising
+        w = ['object.__init__']
+    else:
+        w = ["Passing unrecoginized arguments"]
+    with expected_warnings(w):
+        obj = SuperHasTraits(a=1, b=2)
+    nt.assert_equal(obj.a, 1)
+    assert not hasattr(obj, 'b')
